@@ -7,7 +7,7 @@ from simfempy import fems
 from simfempy import tools
 
 #=================================================================#
-class Elasticity(solvers.solver.Solver):
+class Elasticity(solvers.solver.Application):
     """
     """
     YoungPoisson = {}
@@ -127,7 +127,7 @@ class Elasticity(solvers.solver.Solver):
                 mat[:, i::ncomp, j::ncomp] += (np.einsum('nk,nl->nkl', cellgrads[:, :, i], cellgrads[:, :, j]).T * dV * self.lamcell).T
                 mat[:, i::ncomp, j::ncomp] += (np.einsum('nk,nl->nkl', cellgrads[:, :, j], cellgrads[:, :, i]).T * dV * self.mucell).T
                 mat[:, i::ncomp, i::ncomp] += (np.einsum('nk,nl->nkl', cellgrads[:, :, j], cellgrads[:, :, j]).T * dV * self.mucell).T
-        A = sparse.coo_matrix((mat.flatten(), (rows, cols)), shape=(ncomp*nnodes, ncomp*nnodes)).tocsr()
+        A = sparse.coo_matrix((mat.ravel(), (rows, cols)), shape=(ncomp*nnodes, ncomp*nnodes)).tocsr()
         # if self.method == "sym":
         # rows, cols = A.nonzero()
         # A[cols, rows] = A[rows, cols]
