@@ -4,11 +4,11 @@ import scipy.linalg as linalg
 import scipy.sparse.linalg as splinalg
 from simfempy import solvers
 from simfempy import fems
-import simfempy.tools.analyticalsolution
+import simfempy.tools.analyticalfunction
 import simfempy.tools.iterationcounter
 
 # =================================================================#
-class Stokes(solvers.solver.Solver):
+class Stokes(solvers.solver.Application):
     """
     """
     def generatePoblemData(self, **kwargs):
@@ -18,12 +18,12 @@ class Stokes(solvers.solver.Solver):
     def defineAnalyticalSolution(self, exactsolution, random=True):
         solexact = super().defineAnalyticalSolution(exactsolution=exactsolution, random=random)
         if exactsolution == 'Linear':
-            # solexact.append(simfempy.tools.analyticalsolution.AnalyticalSolution('x+y'))
-            solexact.append(simfempy.tools.analyticalsolution.AnalyticalSolution('0'))
+            # solexact.append(simfempy.tools.analyticalsolution.AnalyticalFunction('x+y'))
+            solexact.append(simfempy.tools.analyticalfunction.AnalyticalSolution3d('0'))
         elif exactsolution == 'Quadratic':
-            # solexact[0] = simfempy.tools.analyticalsolution.AnalyticalSolution('x*x-2*y*x')
-            # solexact[1] = simfempy.tools.analyticalsolution.AnalyticalSolution('-2*x*y+y*y')
-            solexact.append(simfempy.tools.analyticalsolution.AnalyticalSolution('x+y'))
+            # solexact[0] = simfempy.tools.analyticalsolution.AnalyticalFunction('x*x-2*y*x')
+            # solexact[1] = simfempy.tools.analyticalsolution.AnalyticalFunction('-2*x*y+y*y')
+            solexact.append(simfempy.tools.analyticalfunction.AnalyticalSolution3d('x+y'))
         else:
             raise NotImplementedError("unknown function '{}'".format(exactsolution))
         return solexact
@@ -148,7 +148,7 @@ class Stokes(solvers.solver.Solver):
         matA = np.zeros(ncells*nloc*nloc).reshape(ncells, nloc, nloc)
         for i in range(ncomp):
             matA += np.einsum('nk,nl->nkl', cellgrads[:, :, i], cellgrads[:, :, i])
-        matA = ( matA.T*dV*self.mucell).T.flatten()
+        matA = ( matA.T*dV*self.mucell).T.ravel()
         cols = np.tile(facesOfCells, nloc).reshape(ncells, nloc, nloc)
         rows = cols.swapaxes(1, 2)
         A = scipy.sparse.coo_matrix((matA, (rows.reshape(-1), cols.reshape(-1))), shape=(nfaces, nfaces)).tocsr()
