@@ -9,13 +9,9 @@ import meshio
 import numpy as np
 from scipy import sparse
 from simfempy.tools import npext
+from simfempy.tools import timer
 
-
-# try:
-#     import geomdefs
-# except ModuleNotFoundError:
-#     from . import geomdefs
-
+from .testmeshes import __pygmsh6__
 
 #=================================================================#
 class SimplexMesh(object):
@@ -32,7 +28,7 @@ class SimplexMesh(object):
     faces: node ids of faces of shape (nfaces, dimension)
 
     facesOfCells: shape (ncells, dimension+1): contains simplices[i,:]-setminus simplices[i,ii], sorted
-    cellsOfFaces: shape (nfaces, dimension): cellsOfFaces[i,1]=-1 if boundary
+    cellsOfFaces: shape (nfaces, 2): cellsOfFaces[i,1]=-1 if boundary
 
     normals: normal per face of length dS, oriented from  ids of faces of shape (nfaces, dimension)
              normals on boundary are external
@@ -40,85 +36,126 @@ class SimplexMesh(object):
 
     dV: shape (ncells), volumes of simplices
     bdrylabels: dictionary(keys: colors, values: id's of boundary faces)
+    cellsoflabel: dictionary(keys: colors, values: id's of cells)
     """
 
     def __repr__(self):
-        return "SimplexMesh({}): dim/nnodes/ncells/nfaces: {}/{}/{}/{} bdrylabels={}".format(self.geometry, self.dimension, self.nnodes, self.ncells, self.nfaces, list(self.bdrylabels.keys()))
+        s = f"SimplexMesh({self.geometry}): "
+        s += f"dim/nnodes/nfaces/ncells: {self.dimension}/{self.nnodes}/{self.nfaces}/{self.ncells}"
+        s += f"\nbdrylabels={list(self.bdrylabels.keys())}"
+        s += f"\ncellsoflabel={list(self.cellsoflabel.keys())}"
+        return s
+
     def __init__(self, **kwargs):
         if 'mesh' in kwargs:
             self.geometry = 'own'
             mesh = kwargs.pop('mesh')
         else:
-            import pygmsh
-            self.geometry = kwargs.pop('geometry')
-            if 'hmean' in kwargs: hmean = kwargs.pop('hmean')
-            else: hmean = 1
-            self.geometry.define(hmean)
-            # code = self.geometry.get_code()
-            # with open("toto.geo",'w') as file:
-            #     file.write(code)
-            mesh = pygmsh.generate_mesh(self.geometry, verbose=False)
-        self._initMeshPyGmsh(mesh.points, mesh.cells, mesh.cell_data)
-
-    def _checkPhyscalLabels(self, points, cells, celldata):
-        if 'vertex' in cells.keys():
-            if not 'vertex' in celldata.keys() or not 'gmsh:physical' in celldata['vertex'].keys():
-                raise KeyError(f"vertices given without physical_label")
-        if self.dimension == 2:
-            if not 'line' in celldata.keys() or not 'gmsh:physical' in celldata['line'].keys():
-                raise KeyError(f"lines given without physical_label")
-            if not 'triangle' in celldata.keys() or not 'gmsh:physical' in celldata['triangle'].keys():
-                raise KeyError(f"triangles given without physical_label")
-        else:
-            if not 'triangle' in celldata.keys() or not 'gmsh:physical' in celldata['triangle'].keys():
-                raise KeyError(f"triangles given without physical_label")
-            if not 'tetra' in celldata.keys() or not 'gmsh:physical' in celldata['tetra'].keys():
-                raise KeyError(f"tetras given without physical_label")
-
-    def _initMeshPyGmsh(self, points, cells, celldata):
-        if 'tetra' in cells.keys():
-            self.dimension = 3
-        elif 'triangle' in cells.keys():
-            self.dimension = 2
-        else:
-            self.dimension = 1
-        assert points.shape[1] ==3
-        self._checkPhyscalLabels(points, cells, celldata)
-        self.points = points
+            raise KeyError("Needs a mesh (no longer geometry)")
+        self._initMeshPyGmsh(mesh)
+    def _initMeshPyGmsh(self, mesh):
+        # only 3d-coordinates
+        assert mesh.points.shape[1] ==3
+        self.points = mesh.points
         self.nnodes = self.points.shape[0]
-        if 'vertex' in cells.keys():
-            self.vertices = cells['vertex'].reshape(-1)
-            self.vertex_labels = celldata['vertex']['gmsh:physical']
-            verticesoflabel = npext.unique_all(self.vertex_labels)
-            self.verticesoflabel={}
-            for color, ind in zip(verticesoflabel[0], verticesoflabel[1]):
-                self.verticesoflabel[color] = self.vertices[ind]
-        if self.dimension==2:
-            self.simplices = cells['triangle']
-            self._facedata = (cells['line'], celldata['line']['gmsh:physical'])
-            self._constructFacesFromSimplices(cells['line'], celldata['line']['gmsh:physical'])
-            self.cell_labels = celldata['triangle']['gmsh:physical']
+        self.celltypes = [key for key, cellblock in mesh.cells]
+        # for key, cellblock in cells: keys.append(key)
+        # print("self.celltypes", self.celltypes)
+        if 'tetra' in self.celltypes:
+            self.dimension = 3
+            self.simplicesname, self.facesname = 'tetra', 'triangle'
+        elif 'triangle' in self.celltypes:
+            self.dimension = 2
+            self.simplicesname, self.facesname = 'triangle', 'line'
+        elif 'line' in self.celltypes:
+            self.dimension = 1
+            self.simplicesname, self.facesname = 'line', 'vertex'
         else:
-            self.simplices = cells['tetra']
-            self._facedata = (cells['triangle'], celldata['triangle']['gmsh:physical'])
-            self._constructFacesFromSimplices(cells['triangle'], celldata['triangle']['gmsh:physical'])
-            self.cell_labels = celldata['tetra']['gmsh:physical']
-        cellloflabel = npext.unique_all(self.cell_labels)
-        self.cellsoflabel = {}
-        for color, ind in zip(cellloflabel[0], cellloflabel[1]):
-            self.cellsoflabel[color] = ind
-        # for color, ind in zip(cellloflabel[0], cellloflabel[1]):
-        #     print("color", color)
-        #     print("split", cell_labels[ind])
-
+            raise ValueError(f"something wrong {self.celltypes=} {mesh=}")
+        bdryfacesgmshlist = []
+        for key, cellblock in mesh.cells:
+            # print(f"{key=} {cellblock=}")
+            if key == self.simplicesname: self.simplices = cellblock
+            if key == 'vertex': self.vertices = cellblock
+            if key == self.facesname:
+                # print(f"{key=} {cellblock=}")
+                bdryfacesgmshlist.extend(cellblock)
+        if not hasattr(self,"simplices"):
+            raise ValueError(f"something wrong {self.dimension=}")
+        bdryfacesgmsh = np.array(bdryfacesgmshlist)
+        self._constructFacesFromSimplices()
         assert self.dimension+1 == self.simplices.shape[1]
         self.ncells = self.simplices.shape[0]
         self.pointsc = self.points[self.simplices].mean(axis=1)
         self.pointsf = self.points[self.faces].mean(axis=1)
         self._constructNormalsAndAreas()
-        # print(self)
+        if __pygmsh6__:
+            self._initMeshPyGmsh6(mesh.cells, mesh.cell_data['gmsh:physical'], bdryfacesgmsh)
+        else:
+            self._initMeshPyGmsh7(mesh.cells, mesh.cell_sets, bdryfacesgmsh)
 
-    def _constructFacesFromSimplices(self, bdryfacesgmsh, bdrylabelsgmsh):
+    def _initMeshPyGmsh7(self, cells, cell_sets, bdryfacesgmsh):
+        # cell_sets: dict label --> list of None or np.array for each cell_type
+        # the indices of the np.array are not the cellids !
+        # ???
+        # print(f"{cell_sets=}")
+        typesoflabel = {}
+        sizes = {key:0 for key in self.celltypes}
+        cellsoflabel = {key:{} for key in self.celltypes}
+        ctorderd = []
+        for label, cb in cell_sets.items():
+            # print(f"{label=} {cb=}")
+            if len(cb) != len(self.celltypes): raise KeyError(f"mismatch {label=}")
+            for celltype, info in zip(self.celltypes, cb):
+                # only one is supposed to be not None
+                if info is not None:
+                    # print(f"{label=} {type(label)=}")
+                    try: ilabel=int(label)
+                    except: raise ValueError(f"cannot convert to int {label=} {cell_sets=}")
+                    cellsoflabel[celltype][ilabel] = info
+                    sizes[celltype] += info.shape[0]
+                    typesoflabel[ilabel] = celltype
+                    ctorderd.append(celltype)
+        #correcting the numbering in cell_sets
+        n = 0
+        for ct in list(dict.fromkeys(ctorderd)):
+            #eliminates duplicates
+            for l, cb in cellsoflabel[ct].items(): cb -= n
+            n += sizes[ct]
+        self.cellsoflabel = cellsoflabel[self.simplicesname]
+        self.verticesoflabel = {}
+        if self.dimension > 1: self.verticesoflabel = cellsoflabel['vertex']
+        # print(f"{self.verticesoflabel=}")
+        # bdry faces
+        # for key, cellblock in cells:
+        #     if key == self.facesnames[self.dimension - 1]: bdryfacesgmsh = cellblock
+        bdrylabelsgmsh = cellsoflabel[self.facesname]
+        self._constructBoundaryFaces7(bdryfacesgmsh, bdrylabelsgmsh)
+
+    def _initMeshPyGmsh6(self, cells, cdphys, bdryfacesgmsh):
+        if len(cdphys) != len(self.celltypes):
+            raise KeyError(f"not enough physical labels:\n {self.celltypes=}\n {len(cdphys)=}")
+        # first attempt, 'np.append' copies data...
+        _cells = {}
+        _labels = {}
+        for (key, cellblock), cd in zip(cells,cdphys):
+            if len(cellblock) != len(cd):
+                raise ValueError(f"mismatch in {key} {len(cellblock)} {len(cd)}")
+            if not key in _cells.keys():
+                _cells[key] = cellblock
+                _labels[key] = cd
+            else:
+                _cells[key] = np.append(_cells[key], cellblock, axis=0)
+                _labels[key] = np.append(_labels[key], cd, axis=0)
+        self._constructBoundaryFaces6(bdryfacesgmsh, _labels[self.facesname])
+        # self.cellsoflabel = npext.creatdict_unique_all(self.cell_labels)
+        self.cellsoflabel = npext.creatdict_unique_all(_labels[self.simplicesname])
+        self.verticesoflabel = {}
+        if self.dimension > 1 and 'vertex' in _cells.keys():
+            self.vertices = _cells['vertex'].reshape(-1)
+            self.verticesoflabel = npext.creatdict_unique_all(_labels['vertex'])
+
+    def _constructFacesFromSimplices(self):
         simplices = self.simplices
         ncells = simplices.shape[0]
         nnpc = simplices.shape[1]
@@ -130,7 +167,10 @@ class SimplexMesh(object):
         s = "{0}" + (nnpc-2)*", {0}"
         s = s.format(allfaces.dtype)
         order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
-        perm = np.argsort(allfaces.view(s), order=order, axis=0).flatten()
+        if self.dimension==1:
+            perm = np.argsort(allfaces, axis=0).ravel()
+        else:
+            perm = np.argsort(allfaces.view(s), order=order, axis=0).ravel()
         allfacescorted = allfaces[perm]
         self.faces, indices = np.unique(allfacescorted, return_inverse=True, axis=0)
         locindex = np.tile(np.arange(0,nnpc), ncells)
@@ -145,63 +185,106 @@ class SimplexMesh(object):
             self.facesOfCells[cell, loc] = f
             if self.cellsOfFaces[f,0] == -1: self.cellsOfFaces[f,0] = cell
             else: self.cellsOfFaces[f,1] = cell
-        self._constructBoundaryLabels(bdryfacesgmsh, bdrylabelsgmsh)
-    # def _constructCellLabels(self, simpgmsh, labelsgmsh):
-    #     if self.simplices.shape != simpgmsh.shape:
-    #         msg ="wrong shapes self.simplices={} simpgmsh={}".format(self.simplices.shape,simpgmsh.shape)
-    #         raise ValueError(msg)
-    #     simpsorted = np.sort(self.simplices, axis=1)
-    #     labelssorted = np.sort(simpgmsh, axis=1)
-    #     assert simpsorted.shape == labelssorted.shape
-    #     if self.dimension==2:
-    #         dts = "{0}, {0}, {0}".format(simpsorted.dtype)
-    #         dtl = "{0}, {0}, {0}".format(labelssorted.dtype)
-    #         sp = np.argsort(simpsorted.view(dts), order=('f0','f1','f2'), axis=0).flatten()
-    #         lp = np.argsort(labelssorted.view(dtl), order=('f0','f1','f2'), axis=0).flatten()
-    #     else:
-    #         dts = "{0}, {0}, {0}, {0}".format(simpsorted.dtype)
-    #         dtl = "{0}, {0}, {0}, {0}".format(labelssorted.dtype)
-    #         sp = np.argsort(simpsorted.view(dts), order=('f0','f1','f2','f3'), axis=0).flatten()
-    #         lp = np.argsort(labelssorted.view(dtl), order=('f0','f1','f2','f3'), axis=0).flatten()
-    #     spi = np.empty(sp.size, sp.dtype)
-    #     spi[sp] = np.arange(sp.size)
-    #     perm = lp[spi]
-    #     self.cell_labels = labelsgmsh[perm]
-
-    def _constructFaces(self, bdryfacesgmsh, bdrylabelsgmsh):
-        simps, neighbrs = self.delaunay.simplices, self.delaunay.neighbors
-        count=0
-        for i in range(len(simps)):
-            for idim in range(self.dimension+1):
-                if i > neighbrs[i, idim]: count +=1
-        self.nfaces = count
-        self.faces = np.empty(shape=(self.nfaces,self.dimension), dtype=int)
-        self.cellsOfFaces = -1 * np.ones(shape=(self.nfaces, 2), dtype=int)
-        self.facesOfCells = np.zeros(shape=(self.ncells, self.dimension+1), dtype=int)
-        count=0
-        for i in range(len(simps)):
-            for idim in range(self.dimension+1):
-                j = neighbrs[i, idim]
-                if i<j: continue
-                mask = np.array( [ii !=idim for ii in range(self.dimension+1)] )
-                self.faces[count] = np.sort(simps[i,mask])
-                self.facesOfCells[i, idim] = count
-                self.cellsOfFaces[count, 0] = i
-                if j > -1:
-                    for jdim in range(self.dimension+1):
-                        if neighbrs[j, jdim] == i:
-                            self.facesOfCells[j, jdim] = count
-                            self.cellsOfFaces[count, 1] = j
-                            break
-                count +=1
-        # for i in range(len(simps)):
-        #     print("self.facesOfCells {} {}".format(i,self.facesOfCells[i]))
-        # for i in range(self.nfaces):
-        #     print("self.cellsOfFaces {} {}".format(i,self.cellsOfFaces[i]))
+    def _constructBoundaryFaces7(self, bdryfacesgmsh, bdrylabelsgmsh):
         # bdries
-        self._constructBoundaryLabels(bdryfacesgmsh, bdrylabelsgmsh)
+        # bdryfacesgmsh may contains interior edges for len(celllabels)>1
+        bdryfacesgmsh = np.sort(bdryfacesgmsh)
+        bdryids = np.flatnonzero(self.cellsOfFaces[:,1] == -1)
+        # print(f"{bdryids=}")
+        # assert np.all(bdryids == np.flatnonzero(np.any(self.cellsOfFaces == -1, axis=1)))
+        bdryfaces = np.sort(self.faces[bdryids],axis=1)
+        # print(f"{bdryfacesgmsh=}\n{bdryfaces=}")
+        # ind = np.isin(bdryfacesgmsh, bdryfaces)
+        # print(f"{ind=} {bdryfacesgmsh[ind]=}")
+        # print(f"{bdryfaces=}")
+        nbdryfaces = len(bdryids)
+        nnpc = self.simplices.shape[1]
+        s = "{0}" + (nnpc-2)*", {0}"
+        dtb = s.format(bdryfacesgmsh.dtype)
+        dtf = s.format(bdryfaces.dtype)
+        order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
+        if self.dimension==1:
+            bp = np.argsort(bdryfacesgmsh.view(dtb), axis=0).ravel()
+            fp = np.argsort(bdryfaces.view(dtf), axis=0).ravel()
+        else:
+            bp = np.argsort(bdryfacesgmsh.view(dtb), order=order, axis=0).ravel()
+            fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
+        # print(f"{bp=}")
+        # print(f"{fp=}")
 
-    def _constructBoundaryLabels(self, bdryfacesgmsh, bdrylabelsgmsh):
+#https://stackoverflow.com/questions/51352527/check-for-identical-rows-in-different-numpy-arrays
+        indices = (bdryfacesgmsh[bp, None] == bdryfaces[fp]).all(-1).any(-1)
+        # indices = np.isin(bdryfacesgmsh, bdryfaces)
+        # print(f"{indices=}")
+
+        # fp2 = np.searchsorted(bdryfacesgmsh, bdryfaces, sorter=bp)
+        # print(f"{fp2=}")
+
+        # print(f"{bp[indices]=}")
+        # print(f"{bdryfacesgmsh[bp[indices]]=}")
+        # print(f"{bdryfaces[fp]=}")
+        if not np.all(bdryfaces[fp]==bdryfacesgmsh[bp[indices]]):
+            raise ValueError(f"{bdryfaces.T=}\n{bdryfacesgmsh.T=}\n{indices=}\n{bdryfaces[fp].T=}\n{bdryfacesgmsh[bp[indices]].T=}")
+
+        bp2 = bp[indices]
+        for i in range(len(fp)):
+            if not np.all(bdryfacesgmsh[bp2[i]] == bdryfaces[fp[i]]):
+                raise ValueError(f"{i=} {bdryfacesgmsh[bp2[i]]=} {bdryfaces[fp[i]]=}")
+
+        bpi = np.argsort(bp)
+        # bp2i = {bp2[i]:i for i in range(len(bp2))}
+        # print(f"{bp=} \n{bp2=} \n{bpi=} \n{bp2i=} \n{indices=}")
+        binv = -1*np.ones_like(bp)
+        binv[bp2] = np.arange(len(bp2))
+        self.bdrylabels = {}
+        for col, cb in bdrylabelsgmsh.items():
+            # if cb[0] in bp2i.keys():
+            if indices[bpi[cb[0]]]:
+                # for i in range(len(cb)):
+                #     if not bp2i[cb[i]] == binv[cb[i]]:
+                #         raise ValueError(f"{bp2i[cb[i]]} {binv[cb[i]]}")
+                # print(f"{col=}")
+                self.bdrylabels[int(col)] = np.empty_like(cb)
+                # for i in range(len(cb)): self.bdrylabels[int(col)][i] = bdryids[fp[bp2i[cb[i]]]]
+                for i in range(len(cb)): self.bdrylabels[int(col)][i] = bdryids[fp[binv[cb[i]]]]
+            # else:
+            #     assert not indices[bpi[cb[0]]]
+
+
+
+        # nbdrylabelsgmsh = 0
+        # for col, cb in bdrylabelsgmsh.items(): nbdrylabelsgmsh += cb.shape[0]
+        # print(f"{nbdrylabelsgmsh=} {nbdryfaces=}")
+        # # if nbdrylabelsgmsh != nbdryfaces:
+        # #     raise ValueError(f"wrong number of boundary labels {nbdrylabelsgmsh=} != {nbdryfaces=}")
+        # if len(bdryfacesgmsh) != nbdryfaces:
+        #     print("wrong number of bdryfaces {} != {}".format(len(bdryfacesgmsh), nbdryfaces))
+        #     # raise ValueError("wrong number of bdryfaces {} != {}".format(len(bdryfacesgmsh), nbdryfaces))
+        # self.bdrylabels = {}
+        # # colorofbdr = -np.ones(nbdryfaces, dtype=bdryids.dtype)
+        # colorofbdr = -np.ones(nbdrylabelsgmsh, dtype=bdryids.dtype)
+        # for col, cb in bdrylabelsgmsh.items():
+        #     self.bdrylabels[int(col)] = -np.ones( (cb.shape[0]), dtype=np.int32)
+        #     colorofbdr[cb] = col
+        #
+        # # perm = bdryids[bp[fpi]]
+        # # print(f"{perm=}")
+        # # print(f"{colorofbdr=}")
+        # counts = {}
+        # for key in list(self.bdrylabels.keys()): counts[key]=0
+        # for i in range(len(perm)):
+        #     if np.any(bdryfacesgmsh[i] != self.faces[perm[i]]):
+        #         raise ValueError(f"Did not find boundary indices\n{bdryfacesgmsh[i]} {self.faces[perm[i]]}")
+        #     color = colorofbdr[i]
+        #     self.bdrylabels[color][counts[color]] = perm[i]
+        #     counts[color] += 1
+        # # print ("self.bdrylabels", self.bdrylabels)
+        # for col, bl in self.bdrylabels.items():
+        #     print(f"{bl=} {self.bdrylabels2[col]=}")
+        #     # assert np.all(bl == self.bdrylabels2[col])
+
+
+    def _constructBoundaryFaces6(self, bdryfacesgmsh, bdrylabelsgmsh):
         # bdries
         bdryids = np.flatnonzero(self.cellsOfFaces[:,1] == -1)
         assert np.all(bdryids == np.flatnonzero(np.any(self.cellsOfFaces == -1, axis=1)))
@@ -217,16 +300,17 @@ class SimplexMesh(object):
         for i in range(len(colors)):
             self.bdrylabels[colors[i]] = -np.ones( (counts[i]), dtype=np.int32)
         bdryfacesgmsh = np.sort(bdryfacesgmsh)
-        if self.dimension==2:
-            dtb = "{0}, {0}".format(bdryfacesgmsh.dtype)
-            dtf = "{0}, {0}".format(bdryfaces.dtype)
-            bp = np.argsort(bdryfacesgmsh.view(dtb), order=('f0','f1'), axis=0).flatten()
-            fp = np.argsort(bdryfaces.view(dtf), order=('f0','f1'), axis=0).flatten()
+        nnpc = self.simplices.shape[1]
+        s = "{0}" + (nnpc-2)*", {0}"
+        dtb = s.format(bdryfacesgmsh.dtype)
+        dtf = s.format(bdryfaces.dtype)
+        order = ["f0"]+["f{:1d}".format(i) for i in range(1,nnpc-1)]
+        if self.dimension==1:
+            bp = np.argsort(bdryfacesgmsh.view(dtb), axis=0).ravel()
+            fp = np.argsort(bdryfaces.view(dtf), axis=0).ravel()
         else:
-            dtb = "{0}, {0}, {0}".format(bdryfacesgmsh.dtype)
-            dtf = "{0}, {0}, {0}".format(bdryfaces.dtype)
-            bp = np.argsort(bdryfacesgmsh.view(dtb), order=('f0','f1','f2'), axis=0).flatten()
-            fp = np.argsort(bdryfaces.view(dtf), order=('f0','f1','f2'), axis=0).flatten()
+            bp = np.argsort(bdryfacesgmsh.view(dtb), order=order, axis=0).ravel()
+            fp = np.argsort(bdryfaces.view(dtf), order=order, axis=0).ravel()
         bpi = np.empty(bp.size, bp.dtype)
         bpi[bp] = np.arange(bp.size)
         perm = bdryids[fp[bpi]]
@@ -241,12 +325,18 @@ class SimplexMesh(object):
         # print ("self.bdrylabels", self.bdrylabels)
 
     def _constructNormalsAndAreas(self):
-        if self.dimension==2:
+        elem = self.simplices
+        self.sigma = np.array([2 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic)-1 for ic in range(self.ncells)])
+        if self.dimension==1:
+            x = self.points[:,0]
+            self.normals = np.stack((np.ones(self.nfaces), np.zeros(self.nfaces), np.zeros(self.nfaces)), axis=-1)
+            dx1 = x[elem[:, 1]] - x[elem[:, 0]]
+            self.dV = np.abs(dx1)
+        elif self.dimension==2:
             x,y = self.points[:,0], self.points[:,1]
             sidesx = x[self.faces[:, 1]] - x[self.faces[:, 0]]
             sidesy = y[self.faces[:, 1]] - y[self.faces[:, 0]]
             self.normals = np.stack((-sidesy, sidesx, np.zeros(self.nfaces)), axis=-1)
-            elem = self.simplices
             dx1 = x[elem[:, 1]] - x[elem[:, 0]]
             dx2 = x[elem[:, 2]] - x[elem[:, 0]]
             dy1 = y[elem[:, 1]] - y[elem[:, 0]]
@@ -264,7 +354,6 @@ class SimplexMesh(object):
             sidesy = x2*z1 - x1*z2
             sidesz = x1*y2 - x2*y1
             self.normals = 0.5*np.stack((sidesx, sidesy, sidesz), axis=-1)
-            elem = self.simplices
             dx1 = x[elem[:, 1]] - x[elem[:, 0]]
             dx2 = x[elem[:, 2]] - x[elem[:, 0]]
             dx3 = x[elem[:, 3]] - x[elem[:, 0]]
@@ -284,16 +373,12 @@ class SimplexMesh(object):
                 xt = np.mean(self.points[self.simplices[i1]], axis=0) - np.mean(self.points[self.simplices[i0]], axis=0)
                 if np.dot(self.normals[i], xt) < 0:  self.normals[i] *= -1
         # self.sigma = np.array([1.0 - 2.0 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic) for ic in range(self.ncells)])
-        self.sigma = np.array([2 * (self.cellsOfFaces[self.facesOfCells[ic, :], 0] == ic)-1 for ic in range(self.ncells)])
 
     def write(self, filename, dirname = None, point_data=None):
         cell_data_meshio = {}
-
         if hasattr(self,'vertex_labels'):
             cell_data_meshio['vertex'] = {}
             cell_data_meshio['vertex']['gmsh:physical'] = self.vertex_labels
-
-
         if self.dimension ==2:
             cells = {'triangle': self.simplices}
             cells['line'] = self._facedata[0]
